from Globals import DTMLFile
from Globals import InitializeClass
from Products.ZenRelations.RelSchema import *
from Products.ZenModel.ZenossSecurity import ZEN_VIEW, ZEN_CHANGE_DEVICE
from Products.ZenModel.DeviceComponent import DeviceComponent
from Products.ZenModel.ManagedEntity import ManagedEntity

class TerracottaClient(DeviceComponent, ManagedEntity):
    """
    TerracottaClient is the component class representing the Terracotta Client
    """
    meta_type = portal_type = "TerracottaClient"
    
    nodeName = ''
    nodeMbean = ''
    nodeEnabled = True
    nodeConnected = True
    nodeMaxMemory = 0
    nodePort = ''
    status = 1

    _properties = (
        {'id':'nodeName', 'type':'string', 'mode':''},
        {'id':'nodeMbean', 'type':'string', 'mode':''},
        {'id':'nodeEnabled', 'type':'boolean', 'mode':''},
        {'id':'nodeConnected', 'type':'boolean', 'mode':''},
        {'id':'nodePort', 'type':'string', 'mode':''},
        {'id':'nodeMaxMemory', 'type':'float', 'mode':''},
        {'id':'status', 'type':'int', 'mode':''},
        )

    _relations = (
        ('terracottaServer', ToOne(ToManyCont,
            'ZenPacks.community.zenTerracotta.TerracottaServer', 'terracottaClient')),
        )
    
    factory_type_information = ({
        'actions': ({
            'id': 'perfConf',
            'name': 'Template',
            'action': 'objTemplates',
            'permissions': (ZEN_CHANGE_DEVICE,),
        },),
    },)
    
    def viewName(self):
        return self.nodeName
    
    titleOrId = name = viewName

    def primarySortKey(self):
        return self.nodeName

    def getClientDevice(self):
        d = self.dmd.Devices.findDevice(self.nodeName)
        if d:
            return d
        return None
    
    def getClientDeviceLink(self):
        d = self.getClientDevice()
        if d:
            return '<a target="_top" href="%s/devicedetail">%s</a>' % \
                (d.getPrimaryUrlPath(), d.name())
        return ""
        
    def getStatus(self):
        value = self.getClusterConnectionStatusValue()
        if value == 1:
            self.nodeConnected = True
            self.status = 0
        else:
            self.nodeConnected = False
            self.status = 1
        return self.status
    
    def getCacheManagerStatusValue(self):
        return int(self.cacheRRDValue('CacheManagerStatusValue_CacheManagerStatusValue',0))
    
    def getClusterConnectionStatusValue(self):
        return int(self.cacheRRDValue('ClusterConnectionStatusValue_ClusterConnectionStatusValue',0))

    def device(self):
        return self.terracottaServer()
    
    def monitored(self):
        return True

InitializeClass(TerracottaClient)
