#!/usr/bin/env python
###################################
#
#    check-terracotta-client.py
#
###################################
import sys, os, re
from subprocess import *
from time import clock, time
import Globals
from optparse import OptionParser
from Products.ZenUtils.ZenScriptBase import ZenScriptBase
from Products.ZenUtils.Utils import zenPath,prepId
from twisted.internet.utils import getProcessOutput

class TerracottaStatus(ZenScriptBase):
    def __init__(self):
        ZenScriptBase.__init__(self, connect=False)
        self.timeout = 30
        self.exitstatus = 0
        self.message = ''
        
    def buildOptions(self):
        """ program start options
        """
        ZenScriptBase.buildOptions(self)
        self.parser.add_option('--server', dest='server',
            help='Remote server host')
        self.parser.add_option('--jmxport', dest='jmxport',
            help='JMX Port')
        self.parser.add_option('--username', dest='username',
            help='User Name')
        self.parser.add_option('--password', dest='password',
            help='Password')
        
    def deviceSetup(self):
        """ initialize device and start the clock
        """
        startClock = clock()
        self.device = self.options.server
        self.tcpport = self.options.jmxport
        self.username = self.options.username
        self.password = self.options.password
        stopClock = clock()-startClock
        self.checkClock(stopClock,self.timeout,'deviceSetup')

    def checkClock(self,clock,timeout,message):
        """ test that the connection is available
        """
        if clock > timeout:
            self.exitstatus = 2
            self.message = 'CRITICAL:  Connect time limit exceeded'
            print self.message 
            sys.exit(self.exitstatus)
        else:
            self.exitstatus = 0
            self.message = 'OK: Connected to '+self.options.server

    def run(self):
        """ main execution routine
        """
        # set up the device and test the connection
        self.deviceSetup()
        output = self.initialize()
        print self.message
        sys.exit(self.exitstatus)

    def initialize(self):
        """ initiate connection to remote host
        """
        try:
            cachemgrStatus = self.getMetric('CacheManagerStatusValue')
            connectStatus = self.getMetric('ClusterConnectionStatusValue')
            self.message += '|'
            self.message += cachemgrStatus
            self.message += ' '+connectStatus
            self.exitstatus = 0
        except:
            self.message = 'CRITICAL: query failed on port '+self.tcpport
            self.exitstatus = 2

    def getMetric(self,metric):
        """ retrieve the value of a given metric
            in nagios-style format
        """
        mbean = 'atxg:component-name=AppServer,subcomponent-name=EhCacheManager,mbean-name=TerracottaClient'
        result = self.jmxOutput(mbean,metric)
        resultLines = result.split("\n")
        output = resultLines[0]
        attr,value = output.split(metric+': ')
        nagiosOutput = metric+'='+value
        return nagiosOutput

    def jmxOutput(self,mbean,attribute):
        binPath = zenPath('libexec')
        authstring = self.username + ":" + self.password
        deststring = str(self.device) + ":" + str(self.tcpport)
        jarfile = binPath + "/cmdline-jmxclient"
        args = ['/usr/bin/java','-jar',jarfile,authstring,deststring,mbean,attribute]
        return Popen(args,stderr=STDOUT,stdout=PIPE).communicate()[0]
    
if __name__ == "__main__":
    u = TerracottaStatus()
    u.run()
