#!/usr/bin/env python
###################################
#
#    check-terracotta-server.py
#
#
###################################
import sys, os, re
from subprocess import *
from time import clock, time
import Globals
from optparse import OptionParser
from Products.ZenUtils.ZenScriptBase import ZenScriptBase
from Products.ZenUtils.Utils import zenPath,prepId
from twisted.internet.utils import getProcessOutput

class TerracottaStatus(ZenScriptBase):
    def __init__(self):
        ZenScriptBase.__init__(self, connect=False)
        self.timeout = 20
        self.exitStatus = 0
        self.exitMessage = "OK: Command Successful"
        self.isPassive = False
        self.problemExists = False
        
    def deviceSetup(self):
        self.device = self.options.server
        self.tcpport = self.options.jmxport
        self.username = self.options.username
        self.password = self.options.password
            
    def buildOptions(self):
        ZenScriptBase.buildOptions(self)
        self.parser.add_option('--server', dest='server',
            help='Remote server host')
        self.parser.add_option('--jmxport', dest='jmxport',
            help='JMX Port')
        self.parser.add_option('--username', dest='username',
            help='User Name')
        self.parser.add_option('--password', dest='password',
            help='Password')
        
    def run(self):
        """ main execution statement
        """
        self.deviceSetup()
        self.initialize()
        self.quit()

    def initialize(self):
        """ initiate connection to remote host
        """
        self.startClock()
        passive = self.isPassiveStandby()
        self.stopClock()
        
        if self.isPassive == True:
            self.exitMessage = "OK: Server is Passive Standby"
        else:
            self.exitMessage = "OK: Server is Primary"
            try:
                self.startClock()
                clients = self.numClients()
                self.stopClock()
            except:
                clients = 0
            if self.problemExists == False:
                self.exitMessage += "| clients=" + str(clients)
        
    def quit(self):
        """
        exit out Nagios-style
        """
        if self.problemExists == True:
            self.exitStatus = 2
        else:
            self.exitStatus = 0
        print self.exitMessage
        sys.exit(self.exitStatus)
      
    def startClock(self):
        """
        Start the Clock
        """
        self.startWatch = clock()
    
    def stopClock(self):
        """
        Stop the Clock
        """
        self.stopWatch = clock() - self.startWatch
        self.checkClock()
          
    def checkClock(self):
        """ 
        Check whether query exceeds timeout
        """
        if self.stopWatch > self.timeout:
            self.problemExists = True
            self.exitMessage = 'Command timed out, took: ' + str(clock) + ' seconds.'

    def findException(self,line):
        """
        """
        if re.search("Connection refused",line) != None:
            self.problemExists = True
            self.exitMessage = "WARNING: Connection Refused"
        elif re.search("Exception",line) != None:
            self.problemExists = True
            self.exitMessage = "WARNING: Exception encountered"

    def isPassiveStandby(self):
        """ whether this server is a passive standby
        """
        mbean = 'org.terracotta.internal:type=Terracotta Server,name=Terracotta Server'
        attribute = 'PassiveStandby'
        result = self.jmxOutput(mbean,attribute)
        resultLines = result.split("\n")
        for line in resultLines:
            self.findException(line)
            if re.search('true',line) != None :
                self.isPassive =  True
    
    def numClients(self):
        """ find number of currently connected clients
        """
        mbean = 'org.terracotta:type=Terracotta Server,name=DSO'
        attribute = 'ClientLiveObjectCount'
        splitter = 'org.terracotta:name=DSO,type=Terracotta Server,channelID'
        result = self.jmxOutput(mbean,attribute)
        self.findException(result)
        resultLines = result.split("\n")
        output = resultLines[0]
        attr,value = output.split(attribute+': {')
        clients = len(value.split(splitter)) - 1
        return clients

    def jmxOutput(self,mbean,attribute):
        """ return output of jmx query
        """
        binPath = zenPath('bin')
        authstring = self.username + ":" + self.password
        deststring = str(self.device) + ":" + str(self.tcpport)
        jarfile = binPath + "/cmdline-jmxclient"
        args = ['/usr/bin/java','-jar',jarfile,authstring,deststring,mbean,attribute]
        return Popen(args,stderr=STDOUT,stdout=PIPE).communicate()[0]
    
if __name__ == "__main__":
    u = TerracottaStatus()
    u.run()
