from Products.DataCollector.plugins.CollectorPlugin import PythonPlugin
from Products.DataCollector.plugins.CollectorPlugin import CollectorPlugin
from Products.DataCollector.plugins.DataMaps import ObjectMap
from Products.ZenUtils.Utils import zenPath,prepId
from twisted.internet.utils import getProcessOutput
import re,os
from subprocess import *

class TerracottaClientMap(PythonPlugin):
    """Map JMX Client output table to model."""

    relname = "terracottaClient"
    modname = "ZenPacks.community.zenTerracotta.TerracottaClient"
    
    deviceProperties = PythonPlugin.deviceProperties + (
                    'zJmxUsername',
                    'zJmxPassword',
                    'zTerracottaServerPort',
                    )
    
    def queryRemote(self,server,port,username,password,mbean=None,attribute=None):
        binPath = zenPath('libexec')
        authstring = username+":"+password
        deststring = server+":"+str(port)
        jarfile = binPath + "/cmdline-jmxclient"
        args = ['/usr/bin/java','-jar',jarfile,authstring,deststring]
        if mbean != None and attribute != None:
            args = ['/usr/bin/java','-jar',jarfile,authstring,deststring,mbean,attribute]
        output = Popen(args,stderr=STDOUT,stdout=PIPE).communicate()[0]
        return output   
    
    def collect(self, device, log):
        output = self.queryRemote(device.id,
                                  device.zTerracottaServerPort,
                                  device.zJmxUsername,
                                  device.zJmxPassword)
        results = []
        nodes = []
        for line in output.split('\n'):
            findstring = 'org.terracotta:clients=Clients,name=L1 Info Bean'
            if re.search(findstring,line) != None :
                port = None
                nodename = line.split('node=')[1].split('/')[0]
                node = line.split('node=')[1].split(',type=')[0]
                path = line.split('node=')[0]
                type = line.split(',type=')[1]
                mbean = path +'type='+ type +',node='+ node
                environment = self.queryRemote(device.id,
                                            device.zTerracottaServerPort,
                                            device.zJmxUsername,
                                            device.zJmxPassword, 
                                            mbean,
                                            'Environment')
                for e in environment.split('\n'):
                    if re.search('-rmiRegistryPort',e):
                        port = e.split('-rmiRegistryPort ')[1].split(' ')[0]
                
                if port != None:
                    info = {}
                    name = node# + '_' + port
                    print "name",node
                    info['id'] = prepId(name)
                    info['nodeName'] = nodename
                    info['nodeMbean'] = mbean
                    info['nodeEnabled'] = True
                    info['nodeConnected'] = True
                    info['nodePort'] = port
                    meminfo = self.queryRemote(device.id,
                                            device.zTerracottaServerPort,
                                            device.zJmxUsername,
                                            device.zJmxPassword, 
                                            mbean,
                                            'MaxMemory')
                    info['nodeMaxMemory'] = meminfo.split('MaxMemory: ')[1]
                    results.append(info)   
        return results

    def process(self, device, results, log):
        log.info('finding plugin %s for device %s', self.name(), device.id)
        rm = self.relMap()
        for result in results:
            om = self.objectMap(result)
            rm.append(om)
        return rm

